@@ -20,15 +20,15 @@ use std::env;
 
 #[cfg(feature = "compute")]
 fn main() {
-    env_logger::init().unwrap();
+    env_logger::init();
 
-    let identity = openstack::auth::Identity::from_env()
+    let identity = openstack::auth::from_env()
         .expect("Failed to create an identity provider from the environment");
-    let session = openstack::Session::new(identity);
+    let os = openstack::Cloud::new(identity);
+
+    let id = env::args().nth(1).expect("Provide a server ID");
+    let server = os.get_server_by_id(id).expect("Cannot get a server");
 
-    let manager = openstack::compute::v2::servers(&session);
-    let server = manager.get(env::args().nth(1).expect("Provide a server ID"))
-        .expect("Cannot get a server");
     println!("ID = {}, Name = {}, Status = {:?}",
              server.id(), server.name(), server.status());
     println!("Links: flavor = {}, image = {}",
