@@ -18,15 +18,14 @@ extern crate openstack;
 
 #[cfg(feature = "compute")]
 fn main() {
-    env_logger::init().unwrap();
+    env_logger::init();
 
-    let identity = openstack::auth::Identity::from_env()
+    let identity = openstack::auth::from_env()
         .expect("Failed to create an identity provider from the environment");
-    let session = openstack::Session::new(identity);
-    let server_manager = openstack::compute::v2::servers(&session);
-    let sorting = openstack::compute::v2::ServerSortKey::AccessIpv4;
+    let os = openstack::Cloud::new(identity);
+    let sorting = openstack::compute::ServerSortKey::AccessIpv4;
 
-    let servers = server_manager.query()
+    let servers = os.find_servers()
         .sort_by(openstack::Sort::Asc(sorting))
         .fetch().expect("Cannot list servers");
     println!("All servers:");
@@ -34,7 +33,7 @@ fn main() {
         println!("ID = {}, Name = {}", s.id(), s.name());
     }
 
-    let active = server_manager.query()
+    let active = os.find_servers()
         .sort_by(openstack::Sort::Asc(sorting)).with_status("ACTIVE")
         .fetch().expect("Cannot list servers");
     println!("Only active servers:");
