@@ -16,9 +16,10 @@
 
 use std::fmt::Debug;
 
-use reqwest::{Method, RequestBuilder, Url};
+use reqwest::{Method, Url};
 
-use super::super::ApiResult;
+use super::super::Result;
+use super::super::session::RequestBuilder;
 
 
 /// Trait for an authentication method.
@@ -36,14 +37,14 @@ pub trait AuthMethod: BoxedClone + Debug {
     }
 
     /// Region used with this authentication (if any).
-    fn get_region(&self) -> Option<String> { None }
+    fn region(&self) -> Option<String> { None }
 
     /// Get a URL for the requested service.
     fn get_endpoint(&self, service_type: String,
-                    endpoint_interface: Option<String>) -> ApiResult<Url>;
+                    endpoint_interface: Option<String>) -> Result<Url>;
 
     /// Create an authenticated request.
-    fn request(&self, method: Method, url: Url) -> ApiResult<RequestBuilder>;
+    fn request(&self, method: Method, url: Url) -> Result<RequestBuilder>;
 }
 
 
