@@ -15,28 +15,23 @@
 //! OpenStack Identity V3 API support for access tokens.
 
 use std::collections::hash_map::DefaultHasher;
-use std::env;
 use std::fmt;
 use std::hash::{Hash, Hasher};
 
-use reqwest::{Client, IntoUrl, Method, RequestBuilder, Response, StatusCode,
-              Url, UrlError};
+use reqwest::{Client, IntoUrl, Method, Response, StatusCode, Url, UrlError};
 use reqwest::header::{ContentType, Headers};
 
-use super::super::{ApiError, ApiResult};
+use super::super::{Error, Result};
 use super::super::identity::{catalog, protocol};
+use super::super::session::RequestBuilder;
 use super::super::utils::ValueCache;
 use super::AuthMethod;
 
-use ApiError::InvalidInput;
+use Error::InvalidInput;
 
 
 const MISSING_USER: &'static str = "User information required";
 const MISSING_SCOPE: &'static str = "Unscoped tokens are not supported now";
-const MISSING_ENV_VARS: &'static str =
-    "Not all required environment variables were provided";
-const INVALID_ENV_AUTH_URL: &'static str =
-    "Malformed authentication URL provided in the environment";
 const MISSING_SUBJECT_HEADER: &'static str =
     "Missing X-Subject-Token header";
 
@@ -85,18 +80,19 @@ pub struct PasswordAuth {
 
 impl Identity {
     /// Get a reference to the auth URL.
-    pub fn get_auth_url(&self) -> &Url {
+    pub fn auth_url(&self) -> &Url {
         &self.auth_url
     }
 
     /// Create a password authentication against the given Identity service.
-    pub fn new<U>(auth_url: U) -> Result<Identity, UrlError> where U: IntoUrl  {
+    pub fn new<U>(auth_url: U) -> ::std::result::Result<Identity, UrlError>
+            where U: IntoUrl  {
         Identity::new_with_client(auth_url, Client::new())
     }
 
     /// Create a password authentication against the given Identity service.
     pub fn new_with_region<U>(auth_url: U, region: String)
-            -> Result<Identity, UrlError> where U: IntoUrl  {
+            -> ::std::result::Result<Identity, UrlError> where U: IntoUrl  {
         Ok(Identity {
             client: Client::new(),
             auth_url: auth_url.into_url()?,
@@ -108,7 +104,7 @@ impl Identity {
 
     /// Create a password authentication against the given Identity service.
     pub fn new_with_client<U>(auth_url: U, client: Client)
-            -> Result<Identity, UrlError> where U: IntoUrl  {
+            -> ::std::result::Result<Identity, UrlError> where U: IntoUrl  {
         Ok(Identity {
             client: client,
             auth_url: auth_url.into_url()?,
@@ -141,7 +137,7 @@ impl Identity {
     }
 
     /// Create an authentication method based on provided information.
-    pub fn create(self) -> ApiResult<PasswordAuth> {
+    pub fn create(self) -> Result<PasswordAuth> {
         // TODO: support more authentication methods (at least a token)
         let password_identity = match self.password_identity {
             Some(p) => p,
@@ -163,32 +159,6 @@ impl Identity {
         Ok(PasswordAuth::new(self.auth_url, self.region, password_identity,
                              project_scope, self.client))
     }
-
-    /// Create an authentication method from environment variables.
-    pub fn from_env() -> ApiResult<PasswordAuth> {
-        let auth_url = _get_env("OS_AUTH_URL")?;
-        let id = Identity::new(&auth_url).map_err(|_| {
-            InvalidInput(String::from(INVALID_ENV_AUTH_URL))
-        })?;
-
-        let user_name = _get_env("OS_USERNAME")?;
-        let password = _get_env("OS_PASSWORD")?;
-        let project_name = _get_env("OS_PROJECT_NAME")?;
-
-        let user_domain = env::var("OS_USER_DOMAIN_NAME")
-            .unwrap_or(String::from("Default"));
-        let project_domain = env::var("OS_PROJECT_DOMAIN_NAME")
-            .unwrap_or(String::from("Default"));
-
-        id.with_user(user_name, password, user_domain)
-            .with_project_scope(project_name, project_domain)
-            .create()
-    }
-}
-
-#[inline]
-fn _get_env(name: &str) -> ApiResult<String> {
-    env::var(name).or(Err(InvalidInput(String::from(MISSING_ENV_VARS))))
 }
 
 #[inline]
@@ -200,7 +170,7 @@ fn extract_subject_token(headers: &Headers) -> Option<String> {
 
 impl PasswordAuth {
     /// Get a reference to the auth URL.
-    pub fn get_auth_url(&self) -> &Url {
+    pub fn auth_url(&self) -> &Url {
         &self.auth_url
     }
 
@@ -227,7 +197,7 @@ impl PasswordAuth {
         }
     }
 
-    fn token_from_response(&self, resp: Response) -> ApiResult<Token> {
+    fn token_from_response(&self, resp: Response) -> Result<Token> {
         let token_value = match resp.status() {
             StatusCode::Ok | StatusCode::Created => {
                 match extract_subject_token(resp.headers()) {
@@ -236,7 +206,7 @@ impl PasswordAuth {
                         error!("No X-Subject-Token header received from {}",
                                self.token_endpoint);
                         return Err(
-                            ApiError::InvalidResponse(
+                            Error::InvalidResponse(
                                 String::from(MISSING_SUBJECT_HEADER)))
                     }
                 }
@@ -244,12 +214,12 @@ impl PasswordAuth {
             StatusCode::Unauthorized => {
                 error!("Invalid credentials for user {}",
                        self.body.auth.identity.password.user.name);
-                return Err(ApiError::HttpError(resp.status(), resp));
+                return Err(Error::HttpError(resp.status(), resp));
             },
             other => {
                 error!("Unexpected HTTP error {} when getting a token for {}",
                        other, self.body.auth.identity.password.user.name);
-                return Err(ApiError::HttpError(resp.status(), resp));
+                return Err(Error::HttpError(resp.status(), resp));
             }
         };
 
@@ -262,24 +232,24 @@ impl PasswordAuth {
         Ok(Token(token_value))
     }
 
-    fn refresh_token(&self) -> ApiResult<()> {
+    fn refresh_token(&self) -> Result<()> {
         // TODO: refresh on expiration
         self.cached_token.ensure_value(|| {
             debug!("Requesting a token for user {} from {}",
                    self.body.auth.identity.password.user.name,
                    self.token_endpoint);
             let resp = self.client.post(&self.token_endpoint).json(&self.body)
-                .header(ContentType::json()).send()?;
+                .header(ContentType::json()).send()?.error_for_status()?;
             self.token_from_response(resp)
         })
     }
 
-    fn get_token(&self) -> ApiResult<String> {
+    fn get_token(&self) -> Result<String> {
         self.refresh_token()?;
         Ok(self.cached_token.get().unwrap().0)
     }
 
-    fn get_catalog(&self) -> ApiResult<Vec<protocol::CatalogRecord>> {
+    fn get_catalog(&self) -> Result<Vec<protocol::CatalogRecord>> {
         // TODO: catalog caching
         let catalog_url = catalog::get_url(self.auth_url.clone());
         trace!("Requesting a service catalog from {}", catalog_url);
@@ -292,10 +262,10 @@ impl PasswordAuth {
 
 impl AuthMethod for PasswordAuth {
     /// Get region.
-    fn get_region(&self) -> Option<String> { self.region.clone() }
+    fn region(&self) -> Option<String> { self.region.clone() }
 
     /// Create an authenticated request.
-    fn request(&self, method: Method, url: Url) -> ApiResult<RequestBuilder> {
+    fn request(&self, method: Method, url: Url) -> Result<RequestBuilder> {
         let token = self.get_token()?;
         let mut headers = Headers::new();
         // TODO: replace with a typed header
@@ -304,12 +274,12 @@ impl AuthMethod for PasswordAuth {
         {
             let _unused = builder.headers(headers);
         }
-        Ok(builder)
+        Ok(RequestBuilder::new(builder))
     }
 
     /// Get a URL for the requested service.
     fn get_endpoint(&self, service_type: String,
-                    endpoint_interface: Option<String>) -> ApiResult<Url> {
+                    endpoint_interface: Option<String>) -> Result<Url> {
         let real_interface = endpoint_interface.unwrap_or(
             self.default_endpoint_interface());
         debug!("Requesting a catalog endpoint for service '{}', interface \
@@ -324,7 +294,7 @@ impl AuthMethod for PasswordAuth {
             error!("Invalid URL {} received from service catalog for service \
                    '{}', interface '{}' from region {:?}: {}",
                    endp.url, service_type, real_interface, self.region, e);
-            ApiError::InvalidResponse(
+            Error::InvalidResponse(
                 format!("Invalid URL {} for {} - {}",
                         endp.url, service_type, e))
         })
@@ -335,6 +305,7 @@ impl AuthMethod for PasswordAuth {
 pub mod test {
     #![allow(unused_results)]
 
+    use super::super::AuthMethod;
     use super::Identity;
 
     #[test]
@@ -359,7 +330,7 @@ pub mod test {
             .with_project_scope("cool project", "example.com")
             .create().unwrap();
         assert_eq!(&id.auth_url.to_string(), "http://127.0.0.1:8080/identity");
-        assert_eq!(id.get_auth_url().to_string(),
+        assert_eq!(id.auth_url().to_string(),
                    "http://127.0.0.1:8080/identity");
         assert_eq!(&id.body.auth.identity.password.user.name, "user");
         assert_eq!(&id.body.auth.identity.password.user.password, "pa$$w0rd");
@@ -371,6 +342,7 @@ pub mod test {
         assert_eq!(&id.body.auth.scope.project.domain.name, "example.com");
         assert_eq!(&id.token_endpoint,
                    "http://127.0.0.1:8080/identity/v3/auth/tokens");
+        assert_eq!(id.region(), None);
     }
 
     #[test]
