@@ -22,9 +22,7 @@
 //!
 //! 1. Create a suitable authentication method.
 //! 2. Populate it with authentication data (credentials, etc).
-//! 3. Create a [Session](../session/struct.Session.html) by using the
-//!    [Session constructor](../session/struct.Session.html#method.new).
-//! 4. Pass a reference to the resulting session to various API managers.
+//! 3. Create a [Cloud](../struct.Cloud.html).
 //!
 //! # Using password authentication
 //!
@@ -42,35 +40,33 @@
 //! Creating an authentication method using project-scoped tokens:
 //!
 //! ```rust,no_run
-//! use openstack::auth::Identity;
-//! use openstack::Session;
+//! use openstack;
 //!
-//! let auth = Identity::new("https://my.cloud.com/identity").unwrap()
+//! let auth = openstack::auth::Identity::new("https://my.cloud.com/identity").unwrap()
 //!     .with_user("admin", "pa$$w0rd", "My Domain")
 //!     .with_project_scope("project1", "My Domain")
 //!     .create().expect("Failed to authenticate");
-//! let session = Session::new(auth);
+//! let os = openstack::Cloud::new(auth);
 //! ```
 //!
 //! Creating an authentication method from environment variables:
 //!
 //! ```rust,no_run
-//! use openstack::auth::Identity;
-//! use openstack::Session;
+//! use openstack;
 //!
-//! let auth = Identity::from_env().expect("Failed to authenticate");
-//! let session = Session::new(auth);
+//! let auth = openstack::auth::from_env().expect("Failed to authenticate");
+//! let os = openstack::Cloud::new(auth);
 //! ```
 //!
 //! Creating a dummy authentication method for use against clouds that do not
 //! have actual authentication:
 //!
 //! ```
-//! use openstack::auth::NoAuth;
-//! use openstack::Session;
+//! use openstack;
 //!
-//! let auth = NoAuth::new("https://my.cloud.com/some-service").unwrap();
-//! let session = Session::new(auth);
+//! let auth = openstack::auth::NoAuth::new("https://my.cloud.com/some-service")
+//!     .unwrap();
+//! let os = openstack::Cloud::new(auth);
 //! ```
 //!
 //! # Limitations
@@ -84,3 +80,40 @@ mod simple;
 pub use self::base::{AuthMethod, BoxedClone};
 pub use self::simple::NoAuth;
 pub use self::identity::{Identity, PasswordAuth};
+
+use std::env;
+
+use super::Result;
+use super::Error::InvalidInput;
+
+const MISSING_ENV_VARS: &'static str =
+    "Not all required environment variables were provided";
+const INVALID_ENV_AUTH_URL: &'static str =
+    "Malformed authentication URL provided in the environment";
+
+#[inline]
+fn _get_env(name: &str) -> Result<String> {
+    env::var(name).or(Err(InvalidInput(String::from(MISSING_ENV_VARS))))
+}
+
+
+/// Create an authentication method from environment variables.
+pub fn from_env() -> Result<PasswordAuth> {
+    let auth_url = _get_env("OS_AUTH_URL")?;
+    let id = Identity::new(&auth_url).map_err(|_| {
+        InvalidInput(String::from(INVALID_ENV_AUTH_URL))
+    })?;
+
+    let user_name = _get_env("OS_USERNAME")?;
+    let password = _get_env("OS_PASSWORD")?;
+    let project_name = _get_env("OS_PROJECT_NAME")?;
+
+    let user_domain = env::var("OS_USER_DOMAIN_NAME")
+        .unwrap_or(String::from("Default"));
+    let project_domain = env::var("OS_PROJECT_DOMAIN_NAME")
+        .unwrap_or(String::from("Default"));
+
+    id.with_user(user_name, password, user_domain)
+        .with_project_scope(project_name, project_domain)
+        .create()
+}
