@@ -17,10 +17,12 @@
 //! The goal of this project is to provide a simple API for working with
 //! OpenStack clouds.
 //!
-//! # Features
-//!
 //! * [Authentication](auth/index.html)
-//! * Basic [compute API support](compute/index.html)
+//! * [High-level API](struct.Cloud.html)
+//!
+//! API-specific notes:
+//!
+//! * [Compute API support](compute/index.html)
 
 #![crate_name = "openstack"]
 #![crate_type = "lib"]
@@ -29,7 +31,6 @@
 // Taken from https://github.com/rust-unofficial/patterns/
 #![deny(const_err,
         dead_code,
-        extra_requirement_in_impl,
         improper_ctypes,
         legacy_directory_ownership,
         missing_copy_implementations,
@@ -73,17 +74,18 @@ extern crate serde_derive;
 extern crate serde_json;
 
 pub mod auth;
+mod cloud;
 mod common;
 #[cfg(feature = "compute")]
 pub mod compute;
 mod identity;
 pub mod service;
-mod session;
+pub mod session;
 mod utils;
 
-pub use common::ApiError;
-pub use common::ApiResult;
+pub use cloud::Cloud;
+pub use common::Error;
+pub use common::Result;
 pub use common::ApiVersion;
 pub use common::ApiVersionRequest;
 pub use common::Sort;
-pub use session::Session;
