@@ -14,21 +14,15 @@
 
 //! Generic API bits for implementing new services.
 
-use std::marker::PhantomData;
+use std::cmp;
 
-use reqwest::{Method, RequestBuilder, Response, Url};
+use reqwest::Url;
 use reqwest::header::Headers;
-use serde::Serialize;
-use serde::de::DeserializeOwned;
 
-use super::{ApiResult, ApiVersion, ApiVersionRequest, Session};
-use super::utils;
+use super::{Result, ApiVersion, ApiVersionRequest};
+use super::auth::AuthMethod;
 
 
-/// Type of query parameters.
-#[derive(Clone, Debug)]
-pub struct Query(pub Vec<(String, String)>);
-
 /// Information about API endpoint.
 #[derive(Clone, Debug)]
 pub struct ServiceInfo {
@@ -46,142 +40,15 @@ pub trait ServiceType {
     fn catalog_type() -> &'static str;
 
     /// Get basic service information.
-    fn service_info(endpoint: Url, session: &Session)
-        -> ApiResult<ServiceInfo>;
+    fn service_info(endpoint: Url, auth: &AuthMethod) -> Result<ServiceInfo>;
+
+    /// Return headers to set for this API version.
+    fn api_version_headers(_version: ApiVersion) -> Option<Headers> { None }
 }
 
 /// Trait representing a service with API version support.
-pub trait ApiVersioning {
-    /// Return headers to set for this API version.
-    fn api_version_headers(version: ApiVersion) -> ApiResult<Headers>;
-}
+pub trait ApiVersioning {}
 
-/// A service-specific wrapper around Session.
-#[derive(Debug)]
-pub struct ServiceWrapper<'session, Srv: ServiceType> {
-    session: &'session Session,
-    service_type: PhantomData<Srv>,
-    endpoint_interface: Option<String>
-}
-
-
-impl Query {
-    /// Empty query.
-    pub fn new() -> Query {
-        Query(Vec::new())
-    }
-
-    /// Add an item to the query.
-    pub fn push<K, V>(&mut self, param: K, value: V)
-            where K: Into<String>, V: ToString {
-        self.0.push((param.into(), value.to_string()))
-    }
-
-    /// Add a strng item to the query.
-    pub fn push_str<K, V>(&mut self, param: K, value: V)
-            where K: Into<String>, V: Into<String> {
-        self.0.push((param.into(), value.into()))
-    }
-}
-
-impl<'session, Srv: ServiceType> ServiceWrapper<'session, Srv> {
-    /// Create a new wrapper for the specific service.
-    pub fn new(session: &'session Session) -> ServiceWrapper<'session, Srv> {
-        ServiceWrapper {
-            session: session,
-            service_type: PhantomData,
-            endpoint_interface: None
-        }
-    }
-
-    /// Change the endpoint interface used for this wrapper.
-    pub fn with_endpoint_interface(self, endpoint_interface: String)
-            -> ServiceWrapper<'session, Srv> {
-        ServiceWrapper {
-            endpoint_interface: Some(endpoint_interface),
-            .. self
-        }
-    }
-
-    /// Construct and endpoint for the given service from the path.
-    pub fn get_endpoint<P>(&self, path: P, query: Query) -> ApiResult<Url>
-            where P: IntoIterator, P::Item: AsRef<str> {
-        let ep = self.endpoint_interface.clone();
-        let info = self.session.get_service_info::<Srv>(ep)?;
-        let mut url = utils::url::extend(info.root_url, path);
-        let _ = url.query_pairs_mut().extend_pairs(query.0);
-        Ok(url)
-    }
-
-    /// Make an HTTP request to the given service.
-    pub fn request<P>(&self, method: Method, path: P, query: Query)
-            -> ApiResult<RequestBuilder>
-            where P: IntoIterator, P::Item: AsRef<str> {
-        let url = self.get_endpoint(path, query)?;
-        let headers = self.session.service_headers::<Srv>();
-        trace!("Sending HTTP {} request to {} with {:?}",
-               method, url, headers);
-        let mut builder = self.session.request(method, url)?;
-        {
-            let _unused = builder.headers(headers);
-        }
-        Ok(builder)
-    }
-
-    /// Make an HTTP request with JSON body and JSON response.
-    pub fn json<P, Req, Res>(&self, method: Method, path: P, query: Query,
-                             body: &Req) -> ApiResult<Res>
-            where Req: Serialize, Res: DeserializeOwned,
-            P: IntoIterator, P::Item: AsRef<str> {
-        let mut builder = self.request(method, path, query)?;
-        builder.json(body).send()?.error_for_status()?.json().map_err(From::from)
-    }
-
-    /// Make a GET request returning a JSON.
-    pub fn get_json<P, Res>(&self, path: P, query: Query) -> ApiResult<Res>
-            where Res: DeserializeOwned, P: IntoIterator, P::Item: AsRef<str> {
-        self.request(Method::Get, path, query)?.send()?.error_for_status()?
-            .json().map_err(From::from)
-    }
-
-    /// Make a POST request sending and returning a JSON.
-    pub fn post_json<P, Req, Res>(&self, path: P, query: Query, body: &Req)
-            -> ApiResult<Res> where Req: Serialize, Res: DeserializeOwned,
-            P: IntoIterator, P::Item: AsRef<str> {
-        self.json(Method::Post, path, query, body)
-    }
-
-    /// Make a POST request sending and returning a JSON.
-    pub fn put_json<P, Req, Res>(&self, path: P, query: Query, body: &Req)
-            -> ApiResult<Res> where Req: Serialize, Res: DeserializeOwned,
-            P: IntoIterator, P::Item: AsRef<str> {
-        self.json(Method::Put, path, query, body)
-    }
-
-    /// Make a PATCH request sending and returning a JSON.
-    pub fn patch_json<P, Req, Res>(&self, path: P, query: Query, body: &Req)
-            -> ApiResult<Res> where Req: Serialize, Res: DeserializeOwned,
-            P: IntoIterator, P::Item: AsRef<str> {
-        self.json(Method::Patch, path, query, body)
-    }
-
-    /// Make a DELETE request.
-    pub fn delete<P>(&self, path: P, query: Query) -> ApiResult<Response>
-            where P: IntoIterator, P::Item: AsRef<str> {
-        self.request(Method::Delete, path, query)?.send()?
-            .error_for_status().map_err(From::from)
-    }
-}
-
-impl<'session, Srv: ServiceType> Clone for ServiceWrapper<'session, Srv> {
-    fn clone(&self) -> ServiceWrapper<'session, Srv> {
-        ServiceWrapper {
-            session: self.session,
-            service_type: PhantomData,
-            endpoint_interface: self.endpoint_interface.clone()
-        }
-    }
-}
 
 impl ServiceInfo {
     /// Pick an API version.
@@ -192,6 +59,15 @@ impl ServiceInfo {
                 self.minimum_version,
             ApiVersionRequest::Latest =>
                 self.current_version,
+            ApiVersionRequest::LatestFrom(from, to) => {
+                match (self.current_version, self.minimum_version) {
+                    (Some(max), None) if max >= from && max <= to => Some(max),
+                    (None, Some(min)) if min >= from && min <= to => Some(min),
+                    (Some(max), Some(min)) if to >= min && from <= max =>
+                        Some(cmp::min(max, to)),
+                    _ => None
+                }
+            },
             ApiVersionRequest::Exact(req) => {
                 self.current_version.and_then(|max| {
                     match self.minimum_version {
@@ -302,6 +178,78 @@ pub mod test {
         assert!(result.is_none());
     }
 
+    #[test]
+    fn test_pick_version_latest_from_within_range() {
+        let info = service_info(Some(1), Some(24));
+        let req = ApiVersionRequest::LatestFrom(ApiVersion(2, 5),
+                                                ApiVersion(2, 20));
+        let result = info.pick_api_version(req).unwrap();
+        assert_eq!(result, ApiVersion(2, 20));
+    }
+
+    #[test]
+    fn test_pick_version_latest_from_outside_range() {
+        let info = service_info(Some(1), Some(24));
+        let req = ApiVersionRequest::LatestFrom(ApiVersion(2, 0),
+                                                ApiVersion(2, 50));
+        let result = info.pick_api_version(req).unwrap();
+        assert_eq!(result, ApiVersion(2, 24));
+    }
+
+    #[test]
+    fn test_pick_version_latest_from_mismatch_above() {
+        let info = service_info(Some(1), Some(24));
+        let req = ApiVersionRequest::LatestFrom(ApiVersion(2, 25),
+                                                ApiVersion(2, 50));
+        let result = info.pick_api_version(req);
+        assert!(result.is_none());
+    }
+
+    #[test]
+    fn test_pick_version_latest_from_mismatch_below() {
+        let info = service_info(Some(5), Some(24));
+        let req = ApiVersionRequest::LatestFrom(ApiVersion(2, 1),
+                                                ApiVersion(2, 4));
+        let result = info.pick_api_version(req);
+        assert!(result.is_none());
+    }
+
+    #[test]
+    fn test_pick_version_latest_from_only_current() {
+        let info = service_info(None, Some(24));
+        let req = ApiVersionRequest::LatestFrom(ApiVersion(2, 5),
+                                                ApiVersion(2, 50));
+        let result = info.pick_api_version(req).unwrap();
+        assert_eq!(result, ApiVersion(2, 24));
+    }
+
+    #[test]
+    fn test_pick_version_latest_from_only_current_mismatch() {
+        let info = service_info(None, Some(24));
+        let req = ApiVersionRequest::LatestFrom(ApiVersion(2, 5),
+                                                ApiVersion(2, 10));
+        let result = info.pick_api_version(req);
+        assert!(result.is_none());
+    }
+
+    #[test]
+    fn test_pick_version_latest_from_only_minimum() {
+        let info = service_info(Some(1), None);
+        let req = ApiVersionRequest::LatestFrom(ApiVersion(2, 0),
+                                                ApiVersion(2, 50));
+        let result = info.pick_api_version(req).unwrap();
+        assert_eq!(result, ApiVersion(2, 1));
+    }
+
+    #[test]
+    fn test_pick_version_latest_from_only_minimum_mismatch() {
+        let info = service_info(Some(1), None);
+        let req = ApiVersionRequest::LatestFrom(ApiVersion(2, 5),
+                                                ApiVersion(2, 10));
+        let result = info.pick_api_version(req);
+        assert!(result.is_none());
+    }
+
     #[test]
     fn test_pick_version_choice() {
         let info = service_info(Some(1), Some(24));
