@@ -14,6 +14,8 @@
 
 //! Various utilities.
 
+#![allow(dead_code)] // various things are unused with --no-default-features
+
 use std::cell::{Ref, RefCell};
 use std::collections::HashMap;
 use std::fmt::Display;
@@ -23,9 +25,13 @@ use std::str::FromStr;
 use serde::{Deserialize, Deserializer};
 use serde::de::Error as DeserError;
 
-use super::ApiResult;
+use super::Result;
 
 
+/// Type of query parameters.
+#[derive(Clone, Debug)]
+pub struct Query(pub Vec<(String, String)>);
+
 /// Cached clone-able value.
 #[derive(Debug, Clone)]
 pub struct ValueCache<T: Clone>(RefCell<Option<T>>);
@@ -35,6 +41,25 @@ pub struct ValueCache<T: Clone>(RefCell<Option<T>>);
 pub struct MapCache<K: Hash + Eq, V: Clone>(RefCell<HashMap<K, V>>);
 
 
+impl Query {
+    /// Empty query.
+    pub fn new() -> Query {
+        Query(Vec::new())
+    }
+
+    /// Add an item to the query.
+    pub fn push<K, V>(&mut self, param: K, value: V)
+            where K: Into<String>, V: ToString {
+        self.0.push((param.into(), value.to_string()))
+    }
+
+    /// Add a strng item to the query.
+    pub fn push_str<K, V>(&mut self, param: K, value: V)
+            where K: Into<String>, V: Into<String> {
+        self.0.push((param.into(), value.into()))
+    }
+}
+
 impl<T: Clone> ValueCache<T> {
     /// Create a cache.
     pub fn new(value: Option<T>) -> ValueCache<T> {
@@ -42,8 +67,8 @@ impl<T: Clone> ValueCache<T> {
     }
 
     /// Ensure the value is cached.
-    pub fn ensure_value<F>(&self, default: F) -> ApiResult<()>
-            where F: FnOnce() -> ApiResult<T> {
+    pub fn ensure_value<F>(&self, default: F) -> Result<()>
+            where F: FnOnce() -> Result<T> {
         if self.0.borrow().is_some() {
             return Ok(());
         };
@@ -66,8 +91,8 @@ impl<K: Hash + Eq, V: Clone> MapCache<K, V> {
     }
 
     /// Ensure the value is present in the cache.
-    pub fn ensure_value<F>(&self, key: K, default: F) -> ApiResult<()>
-            where F: FnOnce(&K) -> ApiResult<V> {
+    pub fn ensure_value<F>(&self, key: K, default: F) -> Result<()>
+            where F: FnOnce(&K) -> Result<V> {
         if self.0.borrow().contains_key(&key) {
             return Ok(());
         }
@@ -91,8 +116,7 @@ impl<K: Hash + Eq, V: Clone> MapCache<K, V> {
 }
 
 /// Deserialize value where empty string equals None.
-#[allow(dead_code)]
-pub fn empty_as_none<'de, D, T>(des: D) -> Result<Option<T>, D::Error>
+pub fn empty_as_none<'de, D, T>(des: D) -> ::std::result::Result<Option<T>, D::Error>
         where D: Deserializer<'de>, T: FromStr, T::Err: Display {
     let s = String::deserialize(des)?;
     if s.is_empty() {
@@ -106,8 +130,6 @@ pub fn empty_as_none<'de, D, T>(des: D) -> Result<Option<T>, D::Error>
 pub mod url {
     //! Handy primitives for working with URLs.
 
-    #![allow(dead_code)] // unused with --no-default-features
-
     use reqwest::Url;
 
     #[inline]
@@ -141,3 +163,52 @@ pub mod url {
         url
     }
 }
+
+
+#[cfg(test)]
+pub mod test {
+    //! Common primitives for testing.
+
+    use reqwest::{IntoUrl, Url};
+    use reqwest::header::Headers;
+
+    use super::super::{Error, Result, ApiVersion};
+    use super::super::auth::{AuthMethod, NoAuth};
+    use super::super::service::{ServiceInfo, ServiceType};
+    use super::super::session::Session;
+
+    /// Create a session with fake authentication.
+    pub fn new_session<U: IntoUrl>(endpoint: U) -> Session {
+        let auth = NoAuth::new(endpoint).expect("Invalid URL in tests");
+        Session::new(auth)
+    }
+
+    /// Fake service type.
+    pub struct FakeServiceType;
+
+    pub const URL: &'static str = "https://127.0.0.1:5000/";
+
+    impl ServiceType for FakeServiceType {
+        fn catalog_type() -> &'static str { "fake" }
+
+        fn service_info(endpoint: Url, _auth: &AuthMethod) -> Result<ServiceInfo> {
+            if endpoint.port() == Some(5000) {
+                Ok(ServiceInfo {
+                    root_url: Url::parse(URL).unwrap(),
+                    current_version: Some(ApiVersion(1, 42)),
+                    minimum_version: Some(ApiVersion(1, 1)),
+                })
+            } else {
+                Err(Error::EndpointNotFound(String::new()))
+            }
+        }
+
+        fn api_version_headers(version: ApiVersion) -> Option<Headers> {
+            if version >= ApiVersion(1, 1) && version <= ApiVersion(1, 42) {
+                Some(Headers::new())
+            } else {
+                None
+            }
+        }
+    }
+}
